/*  Copyright (C) 2002-2017 Zoltán Nagy
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __duct_concepts_h__
#define __duct_concepts_h__ 1


/*   std includes   */
#include <utility>
#include <type_traits>
#include <complex>


namespace duct {


  /*  Concept of real numbers. 
   *  We have only float, double and long double available.
   */ 
  template<typename _Tp>
  concept bool RealNumbers = std::is_floating_point<_Tp>::value;

  /*   Concept of complex numbers.
   *   We have only the three std::complex variants here.
   */
  template<typename _Tp>
  concept bool ComplexNumbers
  =  std::is_same<std::complex<float>, std::remove_cv_t<_Tp>>::value
  || std::is_same<std::complex<double>, std::remove_cv_t<_Tp>>::value
  || std::is_same<std::complex<long double>, std::remove_cv_t<_Tp>>::value;
  
  
  /*   Concept for real and complex numbers.     */
  template<typename _Tp>
  concept bool ComplexandRealNumbers = RealNumbers<_Tp> || ComplexNumbers<_Tp>;
  
  
  
  namespace traits {
    
    /*  Helper class to unify some properties of the real and complex numbers    */
    template<ComplexandRealNumbers _Tp>
    struct complex_and_real_numbers
    {
      using value_type = typename _Tp::value_type;
    };
    
    
    template<RealNumbers _Tp>
    struct complex_and_real_numbers<_Tp>
    {
      using value_type = _Tp;
    };
  }
  
  
  
  
}   //   namespace duct

#endif


