/*  Copyright (C) 2010-2013 Zoltán Nagy
 *  Created by Zoltan Nagy on 10/5/10.
 *  
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>. 
 */
#ifndef __duct_mapping_h__
#define __duct_mapping_h__ 1


/*   std includes   */
#include <functional>

/*   project headers   */
#include <bits/duct-tags.h>
#include <bits/duct-event.h>



namespace duct {
  
  //========> Some helper fuctions to compute the mappings <=====
  namespace __details {
    
    /*        Elementary color operators    */
    template<std::size_t _Nin>
    void color_a_dagger(int sigma, int emitter, event_base<_Nin>& p)
    {
      static constexpr int QUARK = particle_state::colorlink::quarkend;
      static constexpr int ANTI_QUARK = particle_state::colorlink::antiquarkend;

      int spectator, m = p.template number_of<qcd_out>();
      
      switch(sigma) {
        case 1:
          spectator = p[emitter].color.next;
          p[emitter].color.next  = m;
          p[m].color.prev = emitter;
          p[m].color.next = spectator;
          p[spectator].color.prev = m;
          break;
        case -1:
          spectator = p[emitter].color.prev;
          p[spectator].color.next = m;
          p[m].color.prev = spectator;
          p[m].color.next = emitter;
          p[emitter].color.prev = m;
          break;
        case 2:
          spectator = p[emitter].color.prev;
          p[emitter].color.prev  = QUARK;
          p[spectator].color.next  = m;
          p[m].color.prev = spectator;
          p[m].color.next = ANTI_QUARK;
          break;
        case -2:
          p[p[emitter].color.prev].color.next = p[emitter].color.next;
          p[p[emitter].color.next].color.prev = p[emitter].color.prev;
          p[emitter].color.prev = QUARK;
          p[emitter].color.next  = m;
          p[m].color.prev = emitter;
          p[m].color.next = ANTI_QUARK;
          break;
        case 3:
          spectator = p[emitter].color.next;
          p[emitter].color.next = ANTI_QUARK;
          p[spectator].color.prev = m;
          p[m].color.next = spectator;
          p[m].color.prev = QUARK;
          break;
        case -3:
          p[p[emitter].color.prev].color.next = p[emitter].color.next;
          p[p[emitter].color.next].color.prev = p[emitter].color.prev;
          p[emitter].color.next = ANTI_QUARK;
          p[emitter].color.prev  = m;
          p[m].color.next = emitter;
          p[m].color.prev = QUARK;
          break;
        case 4:
          p[emitter].color.prev  = m;
          p[m].color.next = emitter;
          p[m].color.prev = QUARK;
          break;
        case -4:
          p[emitter].color.next  = m;
          p[m].color.prev = emitter;
          p[m].color.next = ANTI_QUARK;
          break;
        default:
          //   do nothing
          break;
      }
    }
    
    template<std::size_t _Nin>
    void color_a(int sigma, int emitter, event_base<_Nin>& p)
    {
      static constexpr int QUARK = particle_state::colorlink::quarkend;
      static constexpr int ANTI_QUARK = particle_state::colorlink::antiquarkend;
      
      int spectator, m = p.template number_of<qcd_out>();
      
      switch(sigma) {
        case 1:
          spectator = p[emitter].colorP.next;
          p[emitter].colorP.next  = m;
          p[m].colorP.prev = emitter;
          p[m].colorP.next = spectator;
          p[spectator].colorP.prev = m;
          break;
        case -1:
          spectator = p[emitter].colorP.prev;
          p[spectator].colorP.next = m;
          p[m].colorP.prev = spectator;
          p[m].colorP.next = emitter;
          p[emitter].colorP.prev = m;
          break;
        case 2:
          spectator = p[emitter].colorP.prev;
          p[emitter].colorP.prev  = QUARK;
          p[spectator].colorP.next  = m;
          p[m].colorP.prev = spectator;
          p[m].colorP.next = ANTI_QUARK;
          break;
        case -2:
          p[p[emitter].colorP.prev].colorP.next = p[emitter].colorP.next;
          p[p[emitter].colorP.next].colorP.prev = p[emitter].colorP.prev;
          p[emitter].colorP.prev = QUARK;
          p[emitter].colorP.next  = m;
          p[m].colorP.prev = emitter;
          p[m].colorP.next = ANTI_QUARK;
          break;
        case 3:
          spectator = p[emitter].colorP.next;
          p[emitter].colorP.next = ANTI_QUARK;
          p[spectator].colorP.prev = m;
          p[m].colorP.next = spectator;
          p[m].colorP.prev = QUARK;
          break;
        case -3:
          p[p[emitter].colorP.prev].colorP.next = p[emitter].colorP.next;
          p[p[emitter].colorP.next].colorP.prev = p[emitter].colorP.prev;
          p[emitter].colorP.next = ANTI_QUARK;
          p[emitter].colorP.prev  = m;
          p[m].colorP.next = emitter;
          p[m].colorP.prev = QUARK;
          break;
        case 4:
          p[emitter].colorP.prev  = m;
          p[m].colorP.next = emitter;
          p[m].colorP.prev = QUARK;
          break;
        case -4:
          p[emitter].colorP.next  = m;
          p[m].colorP.prev = emitter;
          p[m].colorP.next = ANTI_QUARK;
          break;
        default:
          //  do nothing
          break;
      }
    }
    
  }    //    namespace __details
  
  
  
  /*  Splitting variables
   *  This class is a collection of variables for reconstracting
   *  the event after a splitting in a real splitting.
   */
  struct splitting_vars
  {
    /*  label of the emitter and the possible spectator   */
    int emitter, spectator;
    
    /*  flavor splitting variable   */
    int fl, fm;
    
    /*  momentum splitting variables   */
    double y, z, phi;
    
    /*  color splitting variables   */
    struct color_operation {
      int sigma;
      int emit;
    } color, colorP;
  };
  
  
  
  /*  This functions do the mapping in the color, flavor and momentum space.   */
  template<class _Tag> std::function<lorentzvector<double>(const lorentzvector<double>&)>
  shower_mapping_no_color(event<_Tag>&, const event<_Tag>&, const splitting_vars&);
  
 
  template<class _Tag>
  inline void shower_color_mapping(event<_Tag>& p, const event<_Tag>&, const splitting_vars& vars)
  {
    /*    assigning the new color state    */
    __details::color_a_dagger(vars.color.sigma, vars.color.emit, p);
    __details::color_a(vars.colorP.sigma, vars.colorP.emit, p);
    
    /*    issuing the health report for the new state    */
    p.make_health_report();
  }
  
  
  template<class _Tag>
  inline std::function<lorentzvector<double>(const lorentzvector<double>&)>
  shower_mapping(event<_Tag>& p, const event<_Tag>& q, const splitting_vars& vars)
  {
    /*   mapping momenta and flavors    */
    auto boost = shower_mapping_no_color(p, q, vars);
    
    /*    assigning the new color state    */
    shower_color_mapping(p, q, vars);
    
    return boost;
  }
  
}

#endif

