/*  Copyright (C) 2014 Zoltán Nagy
 *  Created by Zoltán Nagy on 1/28/14.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef __mc_hard_params_h__
#define __mc_hard_params_h__ 1

/*   std includes   */

/*   project headers   */
#include<bits/duct-model.h>
#include<bits/mc-shower-aux.h>
#include<cxx/stackvector.h>




namespace duct {
  
  
  template<class _Type>
  struct hard_params;
  

  
  /*   Parameters of the hard state in the initial state splittings.
   *   It is a very simple structure.
   */
  template<>
  struct hard_params<ini>
  {
    /*   model description   */
    model_ref mdl;
    
    /*   pdf and its integrals   */
    const shower_aux *aux;
    
    /*   actual hard parameters   */
    int fl;
    double mT, eta, Q2, vnull;
    
    /*   index of the emitter parton   */
    int emitter;
    
    /*   spectator variables   */
    struct spectator_vars
    {
      int spectator;
      double zk, xi, xT, cfac;
    };
    
    /*   the max number of spectators in an emission is four   */
    stackvector<spectator_vars, 4> spectators;
    
    /*   veto parameters   */
    double mTmax, mYmax;
  };

  
  /*   Parameters of the hard state in the final state splittings.
   *   It is a very simple structure.
   */
  template<>
  struct hard_params<fin>
  {
    /*   true if it is a decay shower   */
    bool is_decay;
    
    /*   model description   */
    model_ref mdl;
    
    /*   actual hard parameters   */
    int fl;
    double mT, pQ, rl, ymax, al, bl, vnull;
    
    /*   index of the emitter parton   */
    int emitter;

    /*   spectator variables   */
    struct spectator_vars
    {
      int spectator;
      double xi, mu, cfac;
    };
    
    /*   the max number of spactator in an emission is four   */
    stackvector<spectator_vars, 4> spectators;

    /*   veto parameters   */
    double mTmax, mYmax;
  };

}

#endif
