/*  Copyright © 2016 Zoltán Nagy. All rights reserved.
 *  Created by Zoltán Nagy on 04/03/16.
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

/*   std includes   */


/*   project headers   */
#include "bits/mc-hard-state.h"



namespace duct {
  
  namespace __details {
    
    void hard_threshold_weight(shower_stage<dis>& hard, const model_ref& mdl, const event<dis>& p, double q2)
    {
      /*   flavors & momentum fractions   */
      int h = p[hadron{0}].key, f = -p[0].flavor;
      double eta = (p[-1].momentum*p[0].momentum)/(p[-1].momentum*p[hadron{0}].momentum);

      
      /*   threshold weight for parton 0   */
      hard.weights.threshold.main = hard.weights.threshold.reg = hard.weights.threshold.delta = 1.0;

      if(ordering::scheme::kT != mdl.scheme()) {
        hard.weights.threshold.main *=  mdl.hard_thrs_weight(h, f, eta, q2);
      }
      
      /*   PDF ration    */
      const auto& pdf = mdl.pdf(h);
      
      hard.weights.unitary.pdf_ratio = pdf[1].parton(f, eta, q2)/pdf[0].parton(f, eta, q2);
    }
    
    
    
    void hard_threshold_weight(shower_stage<hhc>& hard, const model_ref& mdl, const event<hhc>& p, double q2)
    {
      /*   momentum fractions   */
      double Q2 = p[hadron{-1}].momentum*p[hadron{0}].momentum;
      double eta = (p[-1].momentum*p[hadron{ 0}].momentum)/Q2;
      double etb = (p[ 0].momentum*p[hadron{-1}].momentum)/Q2;
      
      /*   parton flavors   */
      int fa = -p[-1].flavor, fb = -p[0].flavor;
      int ha = p[hadron{-1}].key, hb = p[hadron{ 0}].key;
      
      
      /*   threshold weights     */
      hard.weights.threshold.main = hard.weights.threshold.reg = hard.weights.threshold.delta = 1.0;
      
      if(ordering::scheme::kT != mdl.scheme()) {
        hard.weights.threshold.main *= mdl.hard_thrs_weight(ha, fa, eta, q2);
        hard.weights.threshold.main *= mdl.hard_thrs_weight(hb, fb, etb, q2);
      }
      
      
      /*   PDF ration    */
      
      /*  This is the dirty way, but I don't want to do big changes. 
       *  One should use NLO MSbar PDF in the hard part.
       */
      hard.weights.unitary.pdf_ratio = 1.0;
      
      
      
//      const auto& pdfa = mdl.pdf(ha);
//      const auto& pdfb = (ha == hb ? pdfa : mdl.pdf(hb));
//      
//      hard.weights.unitary.pdf_ratio  = pdfa[1].parton(fa, eta, q2)/pdfa[0].parton(fa, eta, q2);
//      hard.weights.unitary.pdf_ratio *= pdfb[1].parton(fb, etb, q2)/pdfb[0].parton(fb, etb, q2);
    }
  
  }

}
